/**
 * Author: Polo Chau
 * Based on code by Barkin Aygun and Carlos Guestrin
 * Turn a selected face, among all input faces, into a collage of all the faces.
 */
import java.io.File;
import java.io.FilenameFilter;
import imagelib.*;

PImage[] images;
String[] files;
float[] averages;
int[] pixelImages;

// each input image is 50 by 50
int pixelCount = 2500;
int imageDisplayWidth = 100;
int screenWidthAndHeight = 800;


void setup() {


  size(screenWidthAndHeight, screenWidthAndHeight);

  // read list of images from directory
  File datafolder = new File(sketchPath, "data");
  files = datafolder.list(filter);

  // check there are images in the directory
  if (files==null|files.length<1){
    println("You must add images in jpg format to the data subdirectory of your sketch.");
    exit();
  }

  // allocate space for images and color histograms
  images = new PImage[files.length];

  // Read every file and create the array of hue histograms
  for (int i = 0; i < files.length; i++) {
    println(files[i]);
    images[i] = loadImage(files[i]);
  }
  resetImages();


  // calculate image brightness averages
  averages = new float[files.length];
  for (int i=0; i<files.length; i++){

    images[i].loadPixels();
    averages[i] = 0;

    for (int j =0; j<pixelCount; j++){
      averages[i] += brightness(images[i].pixels[j]);
    }

    averages[i] = averages[i] / pixelCount;
  }
}

// replace each pixel of the selected image by one of the input images
// that has the most similar brightness
void drawPixelImages(int index){

  int picy = 0;
  int picx = 0;
  background(0);

  for (int j =0; j<pixelCount; j++){
    if (picx >= 50*15) {
      picx = 0;
      picy += 15;
    }
    image(images[pixelImages[j]],picx,picy,15,15);
    picx += 15;
  }
}



// Redraws all the images on the screen
void resetImages() {
  int picy = 0;
  int picx = 0;
  background(0);
  for (int i = 0; i < images.length ;i++) {
    if (picx >= width) {
      picx = 0;
      picy += imageDisplayWidth;
    }
    image(images[i],picx,picy,imageDisplayWidth,imageDisplayWidth);
    picx += imageDisplayWidth;
  }
}



// when mouse is clicked, figure out what image was selected
void mousePressed() {
  int picIndex;
  if (mouseButton == LEFT) {
    picIndex = (mouseY / imageDisplayWidth) * floor(width/imageDisplayWidth) + mouseX / imageDisplayWidth;
    if (picIndex < images.length) {

      markSimilars(picIndex);
      drawPixelImages(picIndex);
    }
  }
  if (mouseButton == RIGHT){
    resetImages();
  }
}

void draw(){
}


// image "index" was selected
void markSimilars(int index) {
  int rectx, recty;

  // clear previous numberings from images
  resetImages();

  println(index);
  PImage selected = images[index];

  float[] brightnesses = new float[pixelCount];
  pixelImages = new int[pixelCount];


  for (int i = 0; i<pixelCount; i++ ){
    selected.loadPixels();
    brightnesses[i] = brightness( selected.pixels[i] );


    // find cloest match
    int best = 0;
    float smallestDiff = 9999;
    for (int k = 0; k<files.length; k++){
      float thisDiff = abs(averages[k] - brightnesses[i] );
      if ( thisDiff < smallestDiff ){
        smallestDiff = thisDiff;
        best = k;
      }
    }
    pixelImages[i] = best;
    println(pixelImages[i]);
  }
}

// FilenameFilters are used during file retrieval to make sure the program uses only valid images It is used in setup() while listing the files.
FilenameFilter filter = new FilenameFilter() {
  public boolean accept(File dir, String name) {
    if (name.toLowerCase().endsWith(".jpg")) return true;
    return false;
  }
};

